/**
 *
 * @generated d
 *
 **/
#define _TYPE  double
#define _PREC  double
#define _LAMCH LAPACKE_dlamch_work

#define _NAME  "PLASMA_dgeqrf"
/* See Lawn 41 page 120 */
#define _FMULS (n * (2.0 / 3.0 * n + 1.5 ) * n)
#define _FADDS (n * (2.0 / 3.0 * n + 0.5 ) * n)

#include "./timing.c"

static int
RunTest(int *iparam, double *dparam, real_Double_t *t_) 
{
  double *A, *Acpy, *T, *b, *x;
    real_Double_t       t;
    int n     = iparam[TIMING_N];
    int nrhs  = iparam[TIMING_NRHS];
    int check = iparam[TIMING_CHECK];
    int lda = n;
    int ldb = n;

    /* Allocate Data */
    A = (double *)malloc(n*lda*sizeof(double));

    /* Check if unable to allocate memory */
    if ( !A ){
        printf("Out of Memory \n ");
        exit(0);
    }

    /* Initialize Plasma */ 
    PLASMA_Init( iparam[TIMING_THRDNBR] );
    if ( iparam[TIMING_SCHEDULER] )
        PLASMA_Set(PLASMA_SCHEDULING_MODE, PLASMA_DYNAMIC_SCHEDULING );
    else
        PLASMA_Set(PLASMA_SCHEDULING_MODE, PLASMA_STATIC_SCHEDULING );

    /*if ( !iparam[TIMING_AUTOTUNING] ) {*/
        PLASMA_Disable(PLASMA_AUTOTUNING);
        PLASMA_Set(PLASMA_TILE_SIZE,        iparam[TIMING_NB] );
        PLASMA_Set(PLASMA_INNER_BLOCK_SIZE, iparam[TIMING_IB] );
    /* } */
    
    /* Initialiaze Data */
    LAPACKE_dlarnv_work(1, ISEED, n*lda, A);

    /* Allocate Workspace */
    PLASMA_Alloc_Workspace_dgels(n, n, &T);

    /* Save AT in lapack layout for check */
    if ( check ) {
        Acpy = (double *)malloc(lda*n*sizeof(double));
        LAPACKE_dlacpy_work(LAPACK_COL_MAJOR,' ', n, n, A, lda, Acpy, lda);
    }

    t = -cWtime();
    PLASMA_dgeqrf( n, n, A, lda, T );
    t += cWtime();
    *t_ = t;
    
    /* Check the solution */
    if ( check )
      {
        b = (double *)malloc(ldb*nrhs*sizeof(double));
        x = (double *)malloc(ldb*nrhs*sizeof(double));

        LAPACKE_dlarnv_work(1, ISEED, n*nrhs, x);
        LAPACKE_dlacpy_work(LAPACK_COL_MAJOR,' ', n, nrhs, x, ldb, b, ldb);

        PLASMA_dgeqrs(n, n, nrhs, A, lda, T, x, ldb);

        dparam[TIMING_RES] = dcheck_solution(n, n, nrhs, Acpy, lda, b, x, ldb,
                                             &(dparam[TIMING_ANORM]), &(dparam[TIMING_BNORM]), 
                                             &(dparam[TIMING_XNORM]));

        free( Acpy ); 
        free( b ); 
        free( x );
      }

    /* Allocate Workspace */
    free( T );
    free( A );
    PLASMA_Finalize();

    return 0;
}
